#ifndef FREERDP_BUILD_CONFIG_H
#define FREERDP_BUILD_CONFIG_H

#define FREERDP_DATA_PATH "/usr/share/freerdp2"
#define FREERDP_KEYMAP_PATH ""
#define FREERDP_PLUGIN_PATH "lib/freerdp2"

#define FREERDP_INSTALL_PREFIX "/usr"

#define FREERDP_LIBRARY_PATH "lib"

#define FREERDP_ADDIN_PATH "lib/freerdp2"

#define FREERDP_SHARED_LIBRARY_SUFFIX ".dll"
#define FREERDP_SHARED_LIBRARY_PREFIX  "cyg"

#define FREERDP_VENDOR_STRING "FreeRDP"
#define FREERDP_PRODUCT_STRING "FreeRDP"

#endif /* FREERDP_BUILD_CONFIG_H */
